#!/bin/bash -e
#-----------------------------------------------------------------------------
# x11tera.sh
# The script to use with to start or to stop x11tera instance.
#
# Several command line parameters are supported:
#  - start : will cause the x11tera instance to start
#  - desktop : will cause the x11tera instance to start (used on gdm autostart)
#  - daemon : will cause the x11tera instance to start (systemd daemon mode)
#  - stop : will cause the x11tera instance to stop
#  - disconnect : will cause to detach (disconnect) the remote client
#  - teardown : will cause to terminate session (logout) local seat user
#  - vdastart : will cause the tera-vdagent instance to start (use with args)
#  - vdastop : will cause the tera-vdagent instance to stop
#  - track : will cause the x11tera_unlock instance to start (track mode)
#  - untrack : will cause the x11tera_unlock instance to stop (track mode)
#  - siglock : will cause the x11tera_on_lock to start
#  - sigunlock : will cause x11tera_on_unlock to start
#  - controls : will cause the x11tera_unlock instance to start (controls mode)
#  - uncontrols : will cause the x11tera_unlock instance to start (controls mode)
#
# Several environment variables are supported (on 'start' operation):
#  - X11TERA_HIDDEN=yes - will cause the x11tera GUI hide (default)
#  - X11TERA_VISIBLE=yes - will cause the x11tera GUI appear
#  - X11TERA_OPERATION=<operation> - will override mode of operation
#
# Example:
#  X11TERA_VISIBLE=1 x11tera.sh start
#  x11tera.sh stop
#-----------------------------------------------------------------------------

operation=$1
if [ ! -z "$X11TERA_OPERATION" ]; then
  operation=$X11TERA_OPERATION
fi

case "$operation" in
  start|"")
    echo "x11tera START operation in progress..."
    operation=start
    ;;
  desktop)
    echo "x11tera autostart at DESKTOP operation in progress..."
    ;;
  daemon)
    echo "x11tera DAEMON start operation in progress..."
    ;;
  stop)
    echo "x11tera STOP operation in progress..."
    ;;
  disconnect)
    echo "x11tera REMOTE DISCONNECT operation in progress..."
    ;;
  teardown)
    echo "x11tera TEARDOWN (LOGOUT) operation in progress..."
    ;;
  vdastart)
    if [ -z "$2" ] || [ -z "$4" ]; then
      echo "illegal '$operation' operation" 1>&2
      exit 1
    fi
    echo "x11tera VDAGENT STARTUP operation in progress..."
    ;;
  vdastop)
    echo "x11tera VDAGENT STOP operation in progress..."
    ;;
  track)
    echo "x11tera TRACK operation in progress..."
    ;;
  untrack)
    echo "x11tera UNTRACK operation in progress..."
    ;;
  siglock)
    echo "x11tera SIGLOCK operation in progress..."
    ;;
  sigunlock)
    echo "x11tera SIGUNLOCK operation in progress..."
    ;;
  controls)
    echo "x11tera CONTROLS operation in progress..."
    ;;
  uncontrols)
    echo "x11tera UNCONTROLS operation in progress..."
    ;;
  *)
    echo "unsupported '$operation' operation" 1>&2
    exit 1
    ;;
esac


x11tera_is_running()
{
  if [ "0" = "$(pgrep -c -x x11tera)" ]; then
    return 0
  else
    return 1
  fi
}

x11tera_is_autologin_enabled()
{
  display_manager=$1
  if [ "$display_manager" = "lightdm" ]; then
    cfgs=/etc/lightdm/lightdm.conf
    for cfg in $cfgs; do
      if [ -f $cfg ]; then
        if [ ! -z "$(grep '^autologin-user *= *[a-zA-Z_].*' $cfg)" ]; then
          return 1
        fi
      fi
    done
    return 0
  fi
  if [ "$display_manager" = "fly-dm" ]; then
    cfgs=/etc/X11/fly-dm/fly-dmrc
  else  # gdm3
    cfgs="/etc/gdm3/custom.conf /etc/gdm3/daemon.conf"
  fi
  for cfg in $cfgs; do
    if [ -f $cfg ]; then
      if [ ! -z "$(grep '^AutomaticLoginEnable *= *[T|t]rue' $cfg)" ]; then
        return 1
      fi
    fi
  done
  return 0
}

x11tera_is_service_enabled()
{
  if [ "$(systemctl is-active x11tera.service)" = "active" ]; then
    return 1
  elif [ "$(systemctl is-enabled x11tera.service 2>/dev/null)" = "enabled" ]; then
    return 1
  else
    return 0
  fi
}

x11tera_disconnect()
{
  rc=1 ; while read -r tera_pid; do
    if [ ! -z "$tera_pid" ]; then
      echo "sending a client disconnect signal to x11tera with pid=$tera_pid"
      kill -s USR1 $tera_pid && kc=$? || kc=$?
      [ $kc -eq 0 ] && rc=0
    fi
  done <<< $(pgrep -x x11tera)
  return $rc
}

x11tera_teardown()
{
  x11tera_on_teardown && rc=$? || rc=$?
  return $rc
}

x11tera_vdastart()
{
  display=$1
  xauth=$2
  user=$3
  if [ ! -x /usr/bin/tera-vdagent ]; then
    echo "WARNING: tera-vdagent is not installed (resize and clipboard won't work)" 1>&2
  else
    if [ -z ${TERA_VDAGENT_EXTRA_ARGS+x} ] && [ -f /etc/default/tera-vdagent ]; then
     . /etc/default/tera-vdagent
    fi
    if [ -z "$xauth" ]; then
      echo "starting tera-vdagent on $display without authority as $user"
      if [ "$USER" = "root" -o "$UID" = "0" ]; then
        export DISPLAY=$display
        # PLEASE: to setup TERA_VDAGENT_EXTRA_ARGS parameter, do not change this file
        #         please edit /etc/default/tera-vdagent (see manual)
        sudo -n -u $user --preserve-env='DISPLAY' -- /usr/bin/tera-vdagent ${TERA_VDAGENT_EXTRA_ARGS}
      else
        DISPLAY=$display /usr/bin/tera-vdagent ${TERA_VDAGENT_EXTRA_ARGS}
      fi
    else
      echo "starting tera-vdagent on $display with authority '$xauth' as $user"
      if [ "$USER" = "root" -o "$UID" = "0" ]; then
        export DISPLAY=$display
        export XAUTHORITY=$xauth
        # PLEASE: to setup TERA_VDAGENT_EXTRA_ARGS parameter, do not change this file
        #         please edit /etc/default/tera-vdagent (see manual)
        sudo -n -u $user --preserve-env='DISPLAY,XAUTHORITY' -- /usr/bin/tera-vdagent ${TERA_VDAGENT_EXTRA_ARGS}
      else
        DISPLAY=$display XAUTHORITY=$xauth /usr/bin/tera-vdagent ${TERA_VDAGENT_EXTRA_ARGS}
      fi
    fi
  fi
}

x11tera_vdastop()
{
  pgrep -x tera-vdagent | while read -r vdagent_pid; do
    echo "stopping tera-vdagent with pid=$vdagent_pid"
    kill -s TERM $vdagent_pid && echo "tera-vdagent terminated" || echo "not found running tera-vdagent"
  done
}

x11tera_vdadkill()
{
  pgrep -x tera-vdagentd | while read -r vdagentd_pid; do
    echo "stopping tera-vdagentd with pid=$vdagentd_pid"
    kill -s KILL $vdagentd_pid && echo "tera-vdagentd killed" || echo "not found running tera-vdagentd"
  done
}

x11tera_track()
{
  if [ ! -x /usr/bin/x11tera_unlock ]; then
    echo "WARNING: x11tera_unlock not found (unlock won't work)" 1>&2
  else
    echo "starting x11tera_unlock to track user clicks"
    /usr/bin/x11tera_unlock track --all
  fi
}

x11tera_untrack()
{
  pgrep -fl "x11tera_unlock +track" | grep x11tera_unlock | awk '{print $1}' | while read -r terau_pid; do
    echo "stopping x11tera_unlock (track) with pid=$terau_pid"
    kill -s TERM $terau_pid
  done
}

x11tera_controls()
{
  if [ ! -x /usr/bin/x11tera_unlock ]; then
    echo "WARNING: x11tera_unlock not found (unlock won't work)" 1>&2
  else
    echo "starting x11tera_unlock to manage events of lock and unlock desktop"
    /usr/bin/x11tera_unlock controls
  fi
}

x11tera_uncontrols()
{
  pgrep -fl "x11tera_unlock +controls" | grep x11tera_unlock | awk '{print $1}' | while read -r terau_pid; do
    echo "stopping x11tera_unlock (controls) with pid=$terau_pid"
    kill -s TERM $terau_pid
  done
}

x11tera_detect_display_manager()
{
  # проверено: gdm3, slim, /usr/bin/fly-dm
  display_manager="$(cat /etc/X11/default-display-manager | rev | cut -d'/' -f1 | rev)"
  echo "$display_manager display manager has been detected"
}

x11tera_wait_for_Xorg_single_instance()
{
  display_manager=$1
  if [ "$display_manager" != "fly-dm" ]; then
    if [ "$display_manager" != "lightdm" -a "$display_manager" != "gdm3" ]; then
      # проверка кол-ва активных Xorg-сессий (для dm, которые запускают их несколько)
      xorg_num=$(who | awk '{print $1,$2}' | grep "^\w\+ :[0-9]" | wc -l)
      if [ $xorg_num -eq 0 ]; then
        echo "there are no active Xorg sessions with active displays" 1>&2
        return 1
      elif [ $xorg_num -ge 2 ]; then
        #TODO: unsupported (login=user1 display=:1 AND login=user2 display=:2) - how to choose one of?
        echo "too much active Xorg sessions ($xorg_num) with active displays" 1>&2
        return 1
      fi
    fi
    # WARN: several instances of the Xorg may appear here during login (there will be only one :display)
    c=0; while [ $(pgrep -c -x Xorg) -gt 1 ] && [ $c -lt 10 ]; do
      echo "waiting for Xorg single instance ($c)..."
      sleep 0.5
      c=$(($c+1))
    done
  fi
  xorg_num=$(pgrep -c -x Xorg)
  if [ $xorg_num -eq 0 ]; then
    echo "there are no active Xorg sessions (check that Wayland is not being used)" 1>&2
    return 1
  elif [ $xorg_num -ge 2 ]; then
    #TODO: unsupported (login=user1 display=:1 AND login=user2 display=:2) - how to choose one of?
    echo "too much active Xorg sessions ($xorg_num)" 1>&2
    return 1
  fi
  return 0
}

x11tera_setup_internal_variables()
{
  display_manager=$1
  set +e
  xorg_pid=$(pgrep -x Xorg | cut -d' ' -f1)
  xorg_user=$(ps -o user= -p "$xorg_pid")
  xorg_uid=$(ps -o uid= -p "$xorg_pid" | awk '{print $1}')
  cur_system=$(uname -a | awk '{print $4}')
  uname_v=$(uname -v)
  is_ubuntu="$(echo "$uname_v" | grep -oi ubuntu)"
  is_astra="$(echo "$uname_v" | grep -oi astra)"
  [ "$USER" = "root" -o "$UID" = "0" ] && is_root=1 || is_root=0  # from systemd x11tera_controls.service
  [ "$display_manager" = "fly-dm" -a "$USER" = "fly-dm" ] && is_fly=1 || is_fly=0  # from systemd x11tera.service (Astra)
  [ "$display_manager" = "lightdm" -a "$USER" = "$xorg_user" ] && is_ldm=1 || is_ldm=0  # from systemd x11tera.service (SberOS)
  [ "$display_manager" = "gdm3" ] && is_gdm=1 || is_gdm=0  # from systemd x11tera.service (SberOS)
  if [ $is_fly -eq 1 ]; then
    dm_user=fly-dm
  elif [ $is_ldm -eq 1 -o $is_gdm -eq 1 ]; then # SberOS (Debian)
    dm_user=$xorg_user
  elif [ ! -z "$is_ubuntu" ]; then
    dm_user=gdm
  elif [ ! -z "$is_astra" ]; then
    dm_user=fly-dm
  elif [ ! -z "$(cat /etc/passwd | grep "^fly-dm:" | cut -d: -f1)" ]; then
    dm_user=fly-dm
  elif [ ! -z "$(cat /etc/passwd | grep "^gdm:" | cut -d: -f1)" ]; then # Ubuntu
    dm_user=gdm
  else
    dm_user=gdm
  fi
  if [ $is_fly -eq 1 -a "$xorg_user" != "root" -a $is_root -ne 1 ]; then
    echo "x11tera starting as fly display manager 1.8"
    user=fly-dm
    display=":$(pgrep -x Xorg | xargs ps -o cmd= -p | cut -d: -f2 | cut -d' ' -f1)"
    home_dir=""  # don't touch it
  elif [ $is_ldm -eq 1 ]; then
    echo "x11tera starting as light display manager"
    user=$xorg_user
    display=":$(pgrep -x Xorg | xargs ps -o cmd= -p | cut -d: -f2 | cut -d' ' -f1)"
    home_dir=""  # don't touch it
  elif [ $is_gdm -eq 1 ]; then
    echo "x11tera starting as gdm3 display manager"
    user=$xorg_user
    # FIXME: данные находятся в памяти процесса dbus-run-session (greeter)
    #        и в памяти процесса gnome-session-binary (user)
    if [ $xorg_uid -lt 1000 ]; then
      display=:0
    else
      display=:1
    fi
    home_dir=""  # don't touch it
  elif [ $is_root -eq 1 ]; then
    echo "x11tera starting as daemon"
    u=$(who | awk '{print $1,$2}' | grep "^\w\+ :[0-9]")
    if [ ! -z "$u" ]; then
      user=$(echo "$u" | cut -d' ' -f1)
      display=$(echo "$u" | cut -d' ' -f2)
      home_dir=$(eval echo ~$user)
    elif [ "$display_manager" = "fly-dm" ]; then
      user=$xorg_user
      display=":$(pgrep -x Xorg | xargs ps -o cmd= -p | cut -d: -f2 | cut -d' ' -f1)"
      home_dir=""
    else
      echo "the user name and display cannot be determined"
      exit 1
    fi
  else
    echo "x11tera starting as application"
    user=$USER
    display=$DISPLAY
    home_dir=$(eval echo ~$user)
  fi
  xauth=$(pgrep -x Xorg | xargs ps -o cmd= -p | awk -F "-auth." '{print $2}' | cut -d' ' -f1)
  set -e
  echo "current system '$cur_system', dm '$display_manager', dm_user '$dm_user', xorg_user '$xorg_user' ($xorg_uid)"
  echo "current user uid '$UID', user '$USER' => '$user', home '$HOME' => '$home_dir', display '$DISPLAY' => '$display'"
  echo "current xauth '$xauth'"
  return 0
}

x11tera_prepare_xauth_file_to_read()
{
  display_manager=$1
  xauth=$2
  home_dir=$3
  prev_xauth="$xauth"
  if [ "$display_manager" = "fly-dm" ] && [ x"$xauth" != x"" ] && [ "$xorg_user" != "root" ] && [ $is_root -ne 1 ]; then
    echo "starting x11tera with direct ${display_manager} 1.8 authority $xauth"
  elif [ "$display_manager" = "lightdm" ] && [ x"$xauth" != x"" ]; then
    echo "starting x11tera with direct ${display_manager} authority $xauth"
  elif [ "$display_manager" = "gdm3" ] && [ x"$xauth" != x"" ]; then
    echo "starting x11tera with direct ${display_manager} authority $xauth"
  elif [ -z "$home_dir" ]; then
    echo "starting x11tera without authority"
  else
    if [ "$xauth" = "$home_dir/.Xauthority" ]; then
      echo "starting x11tera with same authority"
    elif [ $is_root -eq 0 ]; then
      if [ ! -r "$xauth" ]; then
        echo "WARNING: starting x11tera with unavailable authority" 1>&2
        unset xauth
      else
        echo "starting x11tera with authority copying from '$xauth' to '$home_dir/.Xauthority-tera'"
        cp -fv $xauth $home_dir/.Xauthority-tera
        xauth=$home_dir/.Xauthority-tera
      fi
    else
      echo "starting x11tera with authority substitution of '$xauth' for '$home_dir/.Xauthority-tera'"
      xauth_owner=$(stat -c "%U" $xauth)
      if [ ! -z "$(which sudo)" ]; then
        # direct copy of Xauthority file
        sudo -n -u $xauth_owner cat $xauth | sudo -n -u $user -i tee $home_dir/.Xauthority-tera > /dev/null
      else
        # dirty copy of Xauthority file
        su - $xauth_owner -c "rm -fv /tmp/x11tera_xauth.~ && cp -fv $xauth /tmp/x11tera_xauth.~ && chmod -v 704 /tmp/x11tera_xauth.~"
        su - $user -c "cp -fv /tmp/x11tera_xauth.~ $home_dir/.Xauthority-tera && chmod -v 700 $home_dir/.Xauthority-tera"
        su - $xauth_owner -c "rm -fv /tmp/x11tera_xauth.~"
      fi
      xauth=$home_dir/.Xauthority-tera
    fi
  fi
  [ x"$prev_xauth" != x"$xauth" ] && echo "newest xauth '$xauth'"
  return 0
}

x11tera_siglock()
{
  x11tera_detect_display_manager
  x11tera_wait_for_Xorg_single_instance "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 ] && exit $rc # не удалось найти Xorg в памяти (долго перезапускается?)
  x11tera_setup_internal_variables "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 -o -z "$xorg_pid" ] && exit $rc # что-то с Xorg или другими переменными?
  if [ x"$display_manager" = x"fly-dm" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u fly-dm x11tera_on_lock stage0
  elif [ x"$display_manager" = x"lightdm" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user x11tera_on_lock stage0
  elif [ x"$display_manager" = x"gdm3" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user x11tera_on_lock stage0
  else
    echo "TODO: ... siglock received ..." 1>&2
  fi
  x11tera_on_lock stage1
}

x11tera_sigunlock()
{
  x11tera_on_unlock stage0-systemd
  x11tera_detect_display_manager
  if [ x"$display_manager" = x"lightdm" ]; then
    x11tera_on_unlock stage0-lightdm
  fi
  x11tera_wait_for_Xorg_single_instance "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 ] && exit $rc # не удалось найти Xorg в памяти (долго перезапускается?)
  x11tera_setup_internal_variables "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 -o -z "$xorg_pid" ] && exit $rc # что-то с Xorg или другими переменными?
  if [ x"$display_manager" = x"fly-dm" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u fly-dm x11tera_on_unlock stage1
  elif [ x"$display_manager" = x"lightdm" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user x11tera_on_unlock stage1
  elif [ x"$display_manager" = x"gdm3" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user x11tera_on_unlock stage1
  else
    echo "TODO: ... sigunlock received ..."
  fi
  x11tera_on_unlock stage2
}

x11tera_stop()
{
  #DEBUG: ps ax | grep "x11tera.\+--display" | grep -Ev 'grep|sudo|su +- '
  pgrep -x x11tera | while read -r tera_pid; do
    echo "stopping x11tera with pid=$tera_pid"
    kill -s TERM $tera_pid
  done
  x11tera_detect_display_manager
  if [ "$display_manager" = "gdm3" ]; then
    #x11tera_vdadkill
    x11tera_vdastop
  fi
}

x11tera_daemon()
{
  echo "starting x11tera in daemon mode..."

  x11tera_detect_display_manager
  x11tera_wait_for_Xorg_single_instance "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 ] && exit $rc # приведёт к перезапуску сервиса через N сек
  x11tera_setup_internal_variables "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 -o -z "$xorg_pid" ] && exit $rc # приведёт к перезапуску сервиса через N сек
  x11tera_prepare_xauth_file_to_read "$display_manager" "$xauth" "$home_dir"

  if [ "$display_manager" = "gdm3" ]; then
    DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user x11tera_on_unlock stage1
    if [ $xorg_uid -ge 1000 ]; then
      # TODO: optional logout
      x11tera_on_lock logout
      #(sleep 5 && DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user /usr/bin/tera-vdagent -x --display="$display" ${TERA_VDAGENT_EXTRA_ARGS}) &
    fi
    DISPLAY=$display XAUTHORITY=$xauth sudo -u $xorg_user x11tera --display "$display" --hide ${TERA_X11_EXTRA_ARGS}
  else
    # PLEASE: to setup TERA_X11_EXTRA_ARGS parameter, do not change this file
    #         please edit /etc/default/x11tera (see manual)
    DISPLAY=$display XAUTHORITY=$xauth x11tera --display "$display" --hide ${TERA_X11_EXTRA_ARGS}
  fi
}

if [ "$operation" = "start" ] || [ "$operation" = "desktop" ]; then
  x11tera_detect_display_manager
  x11tera_wait_for_Xorg_single_instance "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 ] && exit $rc
  x11tera_setup_internal_variables "$display_manager" && rc=$? || rc=$?
  [ $rc -ne 0 -o -z "$xorg_pid" ] && exit $rc
  x11tera_prepare_xauth_file_to_read "$display_manager" "$xauth" "$home_dir"

  if [ -z ${TERA_X11_EXTRA_ARGS+x} ] && [ -f /etc/default/x11tera ]; then
   . /etc/default/x11tera
  fi

  if [ $is_root -eq 1 ]; then
    echo "WARNING: x11tera daemon mode whould be used" 1>&2
    exit 1
  elif [ "$operation" = "desktop" ]; then
    # is x11tera running?
    x11tera_is_running && rc=$? || rc=$?
    if [ $rc -eq 1 ]; then
      echo "WARNING: x11tera is already running" 1>&2
      exit 1
    fi
    x11tera_is_service_enabled && rc=$? || rc=$?
    if [ $rc -eq 1 ]; then
      echo "WARNING: x11tera service enabled, x11tera application will not start" 1>&2
      exit 1
    fi
    x11tera_is_autologin_enabled "$display_manager" && rc=$? || rc=$?
    if [ $rc -eq 0 ]; then
      echo "DM autologin disabled, user session will not lock"
    else
      echo "DM autologin enabled, user session will be locked..."
      sleep 5  #TODO: needs to be redone
      loginctl lock-session
    fi
    x11tera --display "$display" --hide ${TERA_X11_EXTRA_ARGS} && rc=$? || rc=$?
    echo "x11tera exited with code=$rc"
    exit $rc
  else
    hidden_gui="--hide"
    if [ ! -z "$X11TERA_HIDDEN" ] && [ "$X11TERA_HIDDEN" = "1" -o "$X11TERA_HIDDEN" = "yes" ]; then
      hidden_gui="--hide"
    elif [ ! -z "$X11TERA_HIDDEN" ] && [ "$X11TERA_HIDDEN" = "0" -o "$X11TERA_HIDDEN" = "no" ]; then
      hidden_gui=""
    elif [ ! -z "$X11TERA_VISIBLE" ] && [ "$X11TERA_VISIBLE" = "1" -o "$X11TERA_VISIBLE" = "yes" ]; then
      hidden_gui=""
    fi
    echo "ATTENTION: don't forget to start tera-vdagent manually"
    x11tera --display "$display" $hidden_gui ${TERA_X11_EXTRA_ARGS} && rc=$? || rc=$?
    echo "x11tera exited with code=$rc"
    exit $rc
  fi
elif [ "$operation" = "daemon" ]; then
  x11tera_daemon
elif [ "$operation" = "stop" ]; then
  x11tera_stop
  #denied:x11tera_vdastop
elif [ "$operation" = "disconnect" ]; then
  x11tera_disconnect && rc=$? || rc=$?
  if [ $rc -eq 1 ]; then
    echo "WARNING: impossible to disconnect remote client" 1>&2
    exit 1
  fi
  #denied:x11tera_vdastop
elif [ "$operation" = "teardown" ]; then
  x11tera_teardown && rc=$? || rc=$?
  if [ $rc -eq 1 ]; then
    echo "WARNING: impossible to logout seat user" 1>&2
    exit 1
  fi
elif [ "$operation" = "vdastart" ]; then
  x11tera_vdastart "$2" "$3" "$4"
elif [ "$operation" = "vdastop" ]; then
  x11tera_vdastop
elif [ "$operation" = "track" ]; then
  x11tera_track
elif [ "$operation" = "untrack" ]; then
  x11tera_untrack
elif [ "$operation" = "siglock" ]; then
  x11tera_siglock
elif [ "$operation" = "sigunlock" ]; then
  x11tera_sigunlock
elif [ "$operation" = "controls" ]; then
  x11tera_controls
elif [ "$operation" = "uncontrols" ]; then
  x11tera_uncontrols
fi

exit 0
